@@ -130,7 +130,7 @@ static void text_handler(GMarkupParseContext *context,
 static void error_handler(GMarkupParseContext *context,
 				GError *error, gpointer user_data)
 {
-	ofono_debug("Error parsing xml response from eppsd: %s\n",
+	DBG("Error parsing xml response from eppsd: %s\n",
 		error->message);
 }
 
@@ -154,7 +154,7 @@ static gint conn_compare_by_cid(gconstpointer a, gconstpointer b)
 }
 
 static struct conn_info *conn_info_create(unsigned int device,
-					  unsigned int channel_id)
+						unsigned int channel_id)
 {
 	struct conn_info *connection = g_try_new0(struct conn_info, 1);
 
@@ -173,26 +173,7 @@ static struct conn_info *conn_info_create(unsigned int device,
  */
 static gboolean caif_if_create(const char *interface, unsigned int connid)
 {
-	int s;
-	static struct ifcaif_param param;
-	static struct ifreq ifr;
-
-	param.ipv4_connid = connid;
-	ifr.ifr_data = (void *) &param;
-	strcpy(ifr.ifr_name, interface);
-
-	s = socket(AF_CAIF, SOCK_SEQPACKET, CAIFPROTO_AT);
-	if (s < 0) {
-		ofono_debug("Failed to create socket for CAIF interface");
-		return FALSE;
-	}
-
-	if (ioctl(s, SIOCCAIFNETNEW, &ifr) < 0) {
-		ofono_debug("Failed to create IP interface for CAIF");
-		return FALSE;
-	}
-
-	return TRUE;
+	return FALSE;
 }
 
 /*
@@ -200,32 +181,7 @@ static gboolean caif_if_create(const char *interface, unsigned int connid)
  */
 static gboolean caif_if_remove(const char *interface, unsigned int connid)
 {
-	int s;
-	static struct ifcaif_param param;
-	static struct ifreq ifr;
-
-	param.ipv4_connid = connid;
-	ifr.ifr_data = (void *) &param;
-	strcpy(ifr.ifr_name, interface);
-
-	s = socket(AF_CAIF, SOCK_SEQPACKET, CAIFPROTO_AT);
-	if (s < 0) {
-		ofono_debug("Failed to create socket for CAIF interface");
-		return FALSE;
-	}
-
-	if (ioctl(s, SIOCGIFINDEX, &ifr) != 0) {
-		ofono_debug("Did not find interface (%s) to remove",
-				interface);
-		return FALSE;
-	}
-
-	if (ioctl(s, SIOCCAIFNETREMOVE, &ifr) < 0) {
-		ofono_debug("Failed to remove IP interface for CAIF");
-		return FALSE;
-	}
-
-	return TRUE;
+	return FALSE;
 }
 
 static void ste_eppsd_down_cb(gboolean ok, GAtResult *result,
@@ -239,8 +195,6 @@ static void ste_eppsd_down_cb(gboolean ok, GAtResult *result,
 	struct conn_info *conn;
 	GSList *l;
 
-	dump_response("ste_eppsd_down_cb", ok, result);
-
 	if (!ok)
 		goto error;
 
@@ -249,7 +203,7 @@ static void ste_eppsd_down_cb(gboolean ok, GAtResult *result,
 				conn_compare_by_cid);
 
 	if (!l) {
-		ofono_debug("Did not find data (used caif device) for"
+		DBG("Did not find data (used caif device) for"
 					"connection with cid; %d",
 					gcd->active_context);
 		goto error;
@@ -258,7 +212,7 @@ static void ste_eppsd_down_cb(gboolean ok, GAtResult *result,
 	conn = l->data;
 
 	if (!caif_if_remove(conn->interface, conn->channel_id)) {
-		ofono_debug("Failed to remove caif interface %s.",
+		DBG("Failed to remove caif interface %s.",
 				conn->interface);
 	}
 
@@ -288,14 +242,12 @@ static void ste_eppsd_up_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct eppsd_response rsp;
 	GMarkupParseContext *context = NULL;
 
-	dump_response("ste_eppsd_up_cb", ok, result);
-
 	l = g_slist_find_custom(g_caif_devices,
 				GUINT_TO_POINTER(gcd->active_context),
 				conn_compare_by_cid);
 
 	if (!l) {
-		ofono_debug("Did not find data (device and channel id)"
+		DBG("Did not find data (device and channel id)"
 					"for connection with cid; %d",
 					gcd->active_context);
 		goto error;
@@ -348,7 +300,7 @@ static void ste_eppsd_up_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	return;
 
 error:
-	ofono_debug("ste_eppsd_up_cb error");
+	DBG("ste_eppsd_up_cb error");
 
 	if (context)
 		g_markup_parse_context_free(context);
@@ -370,8 +322,6 @@ static void ste_cgdcont_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct conn_info *conn;
 	GSList *l;
 
-	dump_response("cgdcont_cb", ok, result);
-
 	if (!ok) {
 		struct ofono_error error;
 
@@ -388,21 +338,21 @@ static void ste_cgdcont_cb(gboolean ok, GAtResult *result, gpointer user_data)
 				conn_compare_by_cid);
 
 	if (!l) {
-		ofono_debug("at_cgdcont_cb, no more available devices");
+		DBG("at_cgdcont_cb, no more available devices");
 		goto error;
 	}
 
 	conn = l->data;
 	conn->cid = gcd->active_context;
-	sprintf(buf, "AT*EPPSD=1,%u,%u", conn->channel_id, conn->cid);
+	snprintf(buf, sizeof(buf), "AT*EPPSD=1,%u,%u",
+			conn->channel_id, conn->cid);
 
 	if (g_at_chat_send(gcd->chat, buf, NULL,
 				ste_eppsd_up_cb, ncbd, g_free) > 0)
 		return;
 
 error:
-	if (ncbd)
-		g_free(ncbd);
+	g_free(ncbd);
 
 	gcd->active_context = 0;
 
@@ -425,26 +375,30 @@ static void ste_gprs_activate_primary(struct ofono_gprs_context *gc,
 	gcd->active_context = ctx->cid;
 	cbd->user = gc;
 
-	/* Set username and password */
-	sprintf(buf, "AT*EIAAUW=%d,1,\"%s\",\"%s\"", ctx->cid,
-		ctx->username, ctx->password);
-
-	if (g_at_chat_send(gcd->chat, buf, none_prefix, NULL, NULL, NULL) == 0)
-		goto error;
-
-	len = sprintf(buf, "AT+CGDCONT=%u,\"IP\"", ctx->cid);
+	len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"", ctx->cid);
 
 	if (ctx->apn)
 		snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
 				ctx->apn);
 
 	if (g_at_chat_send(gcd->chat, buf, none_prefix,
-				ste_cgdcont_cb, cbd, g_free) > 0)
-		return;
+				ste_cgdcont_cb, cbd, g_free) == 0)
+		goto error;
+
+	/*
+	 * Set username and password, this should be done after CGDCONT
+	 * or an error can occur.  We don't bother with error checking
+	 * here
+	 * */
+	snprintf(buf, sizeof(buf), "AT*EIAAUW=%d,1,\"%s\",\"%s\"",
+			ctx->cid, ctx->username, ctx->password);
+
+	g_at_chat_send(gcd->chat, buf, none_prefix, NULL, NULL, NULL);
+
+	return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, NULL, NULL, NULL, NULL, data);
 }
@@ -469,22 +423,21 @@ static void ste_gprs_deactivate_primary(struct ofono_gprs_context *gc,
 				conn_compare_by_cid);
 
 	if (!l) {
-		ofono_debug("at_gprs_deactivate_primary, did not find"
+		DBG("at_gprs_deactivate_primary, did not find"
 			"data (channel id) for connection with cid; %d", id);
 		goto error;
 	}
 
 	conn = l->data;
 
-	sprintf(buf, "AT*EPPSD=0,%u,%u", conn->channel_id, id);
+	snprintf(buf, sizeof(buf), "AT*EPPSD=0,%u,%u", conn->channel_id, id);
 
 	if (g_at_chat_send(gcd->chat, buf, none_prefix,
 				ste_eppsd_down_cb, cbd, g_free) > 0)
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -497,8 +450,6 @@ static void ste_cgact_read_cb(gboolean ok, GAtResult *result,
 	gint cid, state;
 	GAtResultIter iter;
 
-	dump_response("cgact_read_cb", ok, result);
-
 	if (!ok)
 		return;
 
@@ -532,8 +483,6 @@ static void cgev_notify(GAtResult *result, gpointer user_data)
 	GAtResultIter iter;
 	const char *event;
 
-	dump_response("cgev_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CGEV:"))
@@ -560,7 +509,7 @@ static int ste_gprs_context_probe(struct ofono_gprs_context *gc,
 	int i;
 
 	gcd = g_new0(struct gprs_context_data, 1);
-	gcd->chat = chat;
+	gcd->chat = g_at_chat_clone(chat);
 
 	g_at_chat_register(gcd->chat, "+CGEV:", cgev_notify, FALSE, gc, NULL);
 
@@ -584,6 +533,8 @@ static void ste_gprs_context_remove(struct ofono_gprs_context *gc)
 	g_caif_devices = NULL;
 
 	ofono_gprs_context_set_data(gc, NULL);
+
+	g_at_chat_unref(gcd->chat);
 	g_free(gcd);
 }
 
