@@ -45,6 +45,7 @@
 #include <ofono/radio-settings.h>
 #include <ofono/log.h>
 
+#include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
 
 static const char *none_prefix[] = { NULL };
@@ -172,7 +173,7 @@ static int hso_enable(struct ofono_modem *modem)
 	g_at_chat_send(data->control, "ATE0", none_prefix, NULL, NULL, NULL);
 	g_at_chat_send(data->app, "ATE0", none_prefix, NULL, NULL, NULL);
 
-	g_at_chat_send(data->control, "AT+CFUN=1", none_prefix,
+	g_at_chat_send(data->control, "AT+CFUN=4", none_prefix,
 					cfun_enable, modem, NULL);
 
 	return -EINPROGRESS;
@@ -213,6 +214,39 @@ static int hso_disable(struct ofono_modem *modem)
 	return -EINPROGRESS;
 }
 
+static void set_online_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_modem_online_cb_t cb = cbd->cb;
+
+	if (ok)
+		CALLBACK_WITH_SUCCESS(cb, cbd->data);
+	else
+		CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
+static void hso_set_online(struct ofono_modem *modem, ofono_bool_t online,
+				ofono_modem_online_cb_t cb, void *user_data)
+{
+	struct hso_data *data = ofono_modem_get_data(modem);
+	GAtChat *chat = data->control;
+	struct cb_data *cbd = cb_data_new(cb, user_data);
+	char const *command = online ? "AT+CFUN=1" : "AT+CFUN=4";
+
+	DBG("modem %p %s", modem, online ? "online" : "offline");
+
+	if (!cbd)
+		goto error;
+
+	if (g_at_chat_send(chat, command, NULL, set_online_cb, cbd, g_free))
+		return;
+
+error:
+	g_free(cbd);
+
+	CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
 static void hso_pre_sim(struct ofono_modem *modem)
 {
 	struct hso_data *data = ofono_modem_get_data(modem);
@@ -221,13 +255,14 @@ static void hso_pre_sim(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_devinfo_create(modem, 0, "atmodem", data->control);
-	sim = ofono_sim_create(modem, 0, "atmodem", data->control);
+	sim = ofono_sim_create(modem, OFONO_VENDOR_OPTION_HSO,
+				"atmodem", data->control);
 
 	if (sim)
 		ofono_sim_inserted_notify(sim, TRUE);
 }
 
-static void hso_post_sim(struct ofono_modem *modem)
+static void hso_post_online(struct ofono_modem *modem)
 {
 	struct hso_data *data = ofono_modem_get_data(modem);
 	struct ofono_gprs *gprs;
@@ -241,8 +276,10 @@ static void hso_post_sim(struct ofono_modem *modem)
 	ofono_radio_settings_create(modem, 0, "hsomodem", data->app);
 
 	ofono_sms_create(modem, OFONO_VENDOR_OPTION_HSO, "atmodem", data->app);
-	ofono_cbs_create(modem, 0, "atmodem", data->app);
-	ofono_ussd_create(modem, 0, "atmodem", data->app);
+	ofono_cbs_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+				"atmodem", data->app);
+	ofono_ussd_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+				"atmodem", data->app);
 
 	gprs = ofono_gprs_create(modem, 0, "atmodem", data->app);
 	gc = ofono_gprs_context_create(modem, 0, "hsomodem", data->control);
@@ -257,8 +294,9 @@ static struct ofono_modem_driver hso_driver = {
 	.remove		= hso_remove,
 	.enable		= hso_enable,
 	.disable	= hso_disable,
+	.set_online     = hso_set_online,
 	.pre_sim	= hso_pre_sim,
-	.post_sim	= hso_post_sim,
+	.post_online	= hso_post_online,
 };
 
 static int hso_init(void)
