@@ -292,7 +292,8 @@ static void set_cw(struct ofono_call_settings *cs, int new_cw, int mask)
 		else
 			value = "disabled";
 
-		sprintf(buf, "%sCallWaiting", bearer_class_to_string(j));
+		snprintf(buf, sizeof(buf), "%sCallWaiting",
+				bearer_class_to_string(j));
 		ofono_dbus_signal_property_changed(conn, path,
 						OFONO_CALL_SETTINGS_INTERFACE,
 						buf, DBUS_TYPE_STRING,
@@ -313,7 +314,8 @@ static void property_append_cw_conditions(DBusMessageIter *dict,
 		if (!(mask & i))
 			continue;
 
-		sprintf(prop, "%sCallWaiting", bearer_class_to_string(i));
+		snprintf(prop, sizeof(prop), "%sCallWaiting",
+				bearer_class_to_string(i));
 
 		if (conditions & i)
 			value = "enabled";
@@ -370,7 +372,7 @@ static void cw_ss_query_callback(const struct ofono_error *error, int status,
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("setting CW via SS failed");
+		DBG("setting CW via SS failed");
 
 		cs->flags &= ~CALL_SETTINGS_FLAG_CACHED;
 		__ofono_dbus_pending_reply(&cs->pending,
@@ -389,7 +391,7 @@ static void cw_ss_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("setting CW via SS failed");
+		DBG("setting CW via SS failed");
 		__ofono_dbus_pending_reply(&cs->pending,
 					__ofono_error_failed(cs->pending));
 
@@ -525,7 +527,7 @@ static void clip_colp_colr_ss_query_cb(const struct ofono_error *error,
 	const char *value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error occurred during ss control query");
+		DBG("Error occurred during ss control query");
 		__ofono_dbus_pending_reply(&cs->pending,
 					__ofono_error_failed(cs->pending));
 
@@ -609,7 +611,7 @@ static gboolean clip_colp_colr_ss(int type,
 		return TRUE;
 	}
 
-	ofono_debug("Received CLIP/COLR/COLP query ss control");
+	DBG("Received CLIP/COLR/COLP query ss control");
 
 	cs->pending = dbus_message_ref(msg);
 
@@ -625,7 +627,7 @@ static void clir_ss_query_callback(const struct ofono_error *error,
 	const char *value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("setting clir via SS failed");
+		DBG("setting clir via SS failed");
 		__ofono_dbus_pending_reply(&cs->pending,
 					__ofono_error_failed(cs->pending));
 
@@ -673,7 +675,7 @@ static void clir_ss_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("setting clir via SS failed");
+		DBG("setting clir via SS failed");
 		__ofono_dbus_pending_reply(&cs->pending,
 					__ofono_error_failed(cs->pending));
 
@@ -776,6 +778,11 @@ static void cs_unregister_ss_controls(struct ofono_call_settings *cs)
 		__ofono_ussd_ssc_unregister(cs->ussd, "77");
 }
 
+gboolean __ofono_call_settings_is_busy(struct ofono_call_settings *cs)
+{
+	return cs->pending ? TRUE : FALSE;
+}
+
 static DBusMessage *generate_get_properties_reply(struct ofono_call_settings *cs,
 							DBusMessage *msg)
 {
@@ -948,7 +955,7 @@ static DBusMessage *cs_get_properties(DBusConnection *conn, DBusMessage *msg,
 {
 	struct ofono_call_settings *cs = data;
 
-	if (cs->pending)
+	if (cs->pending || __ofono_ussd_is_busy(cs->ussd))
 		return __ofono_error_busy(msg);
 
 	if (cs->flags & CALL_SETTINGS_FLAG_CACHED)
@@ -994,7 +1001,7 @@ static void clir_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("setting clir failed");
+		DBG("setting clir failed");
 		__ofono_dbus_pending_reply(&cs->pending,
 					__ofono_error_failed(cs->pending));
 
@@ -1056,7 +1063,7 @@ static void cw_set_callback(const struct ofono_error *error, void *data)
 	struct ofono_call_settings *cs = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error occurred during CW set");
+		DBG("Error occurred during CW set");
 
 		__ofono_dbus_pending_reply(&cs->pending,
 					__ofono_error_failed(cs->pending));
@@ -1125,7 +1132,7 @@ static DBusMessage *cs_set_property(DBusConnection *conn, DBusMessage *msg,
 	const char *property;
 	int cls;
 
-	if (cs->pending)
+	if (cs->pending || __ofono_ussd_is_busy(cs->ussd))
 		return __ofono_error_busy(msg);
 
 	if (!dbus_message_iter_init(msg, &iter))
